Imports System.ComponentModel
Imports System.Windows.Forms

<ToolboxBitmap(GetType(HoverStatus), "HoverStatus_tool.bmp"), _
 ProvideProperty("HoverTip", GetType(Control))> _
Public Class HoverStatus
    Inherits System.Windows.Forms.StatusBar
    Implements IExtenderProvider

    ' Create the first panel (where we will show the status).
    Public Sub New()
        MyBase.New()

        ' Make the first panel where we will display status.
        With Me.Panels.Add("")
            .AutoSize = StatusBarPanelAutoSize.Spring
            .Style = StatusBarPanelStyle.Text
        End With

        Me.ShowPanels = True
    End Sub

    ' Stores information about our client controls.
    Private m_ClientInfo As New Hashtable

    ' Information about a client control.
    Private Class ClientInfo
        Public ClientControl As Control
        Public HoverTip As String = ""

        ' Save a reference to the client.
        Public Sub New(ByVal client_control As Control)
            ClientControl = client_control
        End Sub

        ' Return True if the control has a default blank HoverTip.
        Public Function IsDefault() As Boolean
            Return HoverTip = ""
        End Function
    End Class

    ' We can extend any control.
    Public Function CanExtend(ByVal extendee As Object) As Boolean Implements System.ComponentModel.IExtenderProvider.CanExtend
        Return TypeOf extendee Is Control
    End Function

    ' Return this control's HoverTip.
    <Category("Data"), _
     DefaultValue("")> _
    Public Function GetHoverTip(ByVal client_control As Control) As String
        ' Get the control's ClientInfo if it exists.
        Return GetClientInfo(client_control).HoverTip
    End Function

    ' Set this control's HoverTip.
    <Category("Data"), _
     DefaultValue("")> _
    Public Sub SetHoverTip(ByVal client_control As Control, ByVal tool_tip As String)
        ' Get this control's ClientInfo.
        Dim client_info As ClientInfo = GetClientInfo(client_control)

        ' Set the new value.
        client_info.HoverTip = tool_tip

        ' Add or remove the ClientInfo if necessary.
        AddOrRemoveIfNecessary(client_control, client_info)
    End Sub

    ' Return this control's ClientInfo or a new blank one.
    Private Function GetClientInfo(ByVal client_control As Control) As ClientInfo
        ' See if we have ClientInfo for this control.
        If m_ClientInfo.Contains(client_control) Then
            ' We have ClientInfo for this control. Return it.
            Return DirectCast(m_ClientInfo(client_control), ClientInfo)
        Else
            ' We do not have ClientInfo for this control.
            ' Return a new ClientInfo with a default blank string.
            Return New ClientInfo(client_control)
        End If
    End Function

    ' Add or remove this ClientInfo if necessary.
    Private Sub AddOrRemoveIfNecessary(ByVal client_control As Control, ByVal client_info As ClientInfo)
        ' See if the ClientInfo should be present but is not,
        ' or should not be present but is.
        If client_info.IsDefault Then
            ' The ClientInfo should not be present.
            If m_ClientInfo.Contains(client_control) Then
                ' The ClientInfo is present but should not be.
                m_ClientInfo.Remove(client_control)
                RemoveHandler client_control.MouseEnter, AddressOf Client_MouseEnter
                RemoveHandler client_control.MouseLeave, AddressOf Client_MouseLeave
            End If
        Else
            ' The ClientInfo should be present.
            If Not m_ClientInfo.Contains(client_control) Then
                ' The ClientInfo is missing but should be present.
                m_ClientInfo.Add(client_control, client_info)
                AddHandler client_control.MouseEnter, AddressOf Client_MouseEnter
                AddHandler client_control.MouseLeave, AddressOf Client_MouseLeave
            End If
        End If
    End Sub

    ' Handles client MouseEnter events.
    Private Sub Client_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs)
        ' Get the client's ClientInfo.
        Dim client_info As ClientInfo = GetClientInfo(sender)

        ' Show the focus tip.
        Me.Panels(0).Text = client_info.HoverTip
    End Sub

    ' Handles client MouseLeave events.
    Private Sub Client_MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs)
        ' Clear the focus tip.
        Me.Panels(0).Text = ""
    End Sub
End Class
